/*
 *	Network Queueing System (NQS)
 *  This version of NQS is Copyright (C) 1992  John Roman
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 1, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/*
*  PROJECT:     Network Queueing System
*  AUTHOR:      John Roman
*
*  Modification history:
*
*       Version Who     When            Description
*       -------+-------+---------------+-------------------------
*       V01.10  JRR                     Initial version.
*       V01.20  JRR     16-Jan-1992	Added support for RS6000.
*	V01.3	JRR	17-Jun-1992	Added header.
*	V01.4	JRR	11-Nov-1992	Added support for C prototypes.
*	V01.5	JRR	03-Mar-1994	Fixed up NMAP.
*/
/*++ fopendata.c - Network Queueing System
 *
 * $Source: /usr2/jrroma/nqs/nqs-3.35.6/lib/RCS/fopendata.c,v $
 *
 * DESCRIPTION:
 *
 *	Open an NQS req data file as called from a server returning
 *	the opened file stream.
 *
 *
 *	Author:
 *	-------
 *	Brent A. Kingsbury, Sterling Software Incorporated.
 *	August 12, 1985.
 *
 *
 * STANDARDS VIOLATIONS:
 *   None.
 *
 * REVISION HISTORY: ($Revision: 1.5 $ $Date: 1994/03/30 20:32:29 $ $State: Exp $)
 * $Log: fopendata.c,v $
 * Revision 1.5  1994/03/30  20:32:29  jrroma
 * Version 3.35.6
 *
 * Revision 1.4  92/12/22  15:46:01  jrroma
 * Version 3.30
 * 
 * Revision 1.3  92/06/18  13:23:49  jrroma
 * Added gnu header
 * 
 * Revision 1.2  92/01/16  15:45:37  jrroma
 * Added support for RS6000.
 * 
 * Revision 1.1  92/01/16  15:44:46  jrroma
 * Initial revision
 * 
 *
 */

#include <stdio.h>
#include <sys/types.h>
#include "nmap.h"				/* Get definition of mid_t */

extern int opendata();			/* Open NQS req data file */

/*** fopendata
 *
 *
 *	FILE *fopendata():
 *
 *	Open an NQS req data file as called from a server returning
 *	the opened file stream.
 *
 *	Note that the current directory of the caller must be the
 *	NQS root directory (which is the directory at the time that
 *	the server is exec'd from the NQS daemon).
 *
 *	Returns:
 *		The opened file stream if successful, otherwise
 *		NULL is returned.
 */
FILE *fopendata (orig_seqno, orig_mid, datano)
long orig_seqno;			/* Sequence number for req */
Mid_t orig_mid;				/* Machine-id for req */
int datano;				/* Data file number */
{
	int fd;				/* Opened file descr */

	if ((fd = opendata (orig_seqno, orig_mid, datano)) != -1) {
		return (fdopen (fd, "r"));
	}
	return (NULL);
}
